import { g_CurrentContext } from './Scope';
import { addToUpdateQueue } from './UpdateQueue';
export class tswPropVal {
    ctxs;
    insideSet = false;
    dbg_name;
    val;
    onChanged;
    constructor(initialValue, name) {
        this.val = initialValue;
        this.dbg_name = name;
    }
    dbg_ctxs() {
        return this.ctxs;
    }
    ctxRemove(ctx) {
        if (this.ctxs)
            this.ctxs.delete(ctx);
    }
    ctxAttach() {
        const ctx = g_CurrentContext.getCurrent();
        if (ctx) {
            ctx.addPropVal(this);
            if (!this.ctxs)
                this.ctxs = new Set();
            this.ctxs.add(ctx);
        }
    }
    get() {
        this.ctxAttach();
        return this.val;
    }
    set(v) {
        if (this.insideSet)
            return false;
        this.insideSet = true;
        try {
            if (this.val !== v) {
                this.val = v;
                this.updateContexts();
                if (this.onChanged)
                    this.onChanged();
                return true;
            }
            else {
                return false;
            }
        }
        finally {
            this.insideSet = false;
        }
    }
    updateContexts() {
        const ctxs = this.ctxs;
        this.ctxs = null;
        if (ctxs)
            addToUpdateQueue(ctxs);
    }
    isTrue(contentTrue, contentFalse) {
        return () => this.get() ? contentTrue : contentFalse;
    }
    isFalse(content) {
        return () => !this.get() && content;
    }
    isEqual(val, content) {
        return () => this.get() == val && content;
    }
    convert(converter) {
        return {
            get: () => converter.to(this.get()),
            set: v => this.set(converter.from(v)),
        };
    }
    convert2(to, from) {
        return {
            get: () => to(this.get()),
            set: v => this.set(from(v)),
        };
    }
    to(to) {
        return {
            get: () => to(this.get()),
        };
    }
}
export class tswPropValArray extends tswPropVal {
    constructor(items) {
        super(items);
    }
    getItem(index) {
        const a = this.get();
        return a[index];
    }
    addItem(item, index) {
        const a = this.val;
        if (index == null) {
            a.push(item);
        }
        else {
            a.splice(index, 0, item);
        }
        this.updateContexts();
    }
    delItem(item) {
        this.removeItem(item);
    }
    removeItem(item) {
        const index = this.val.indexOf(item);
        this.removeItemAt(index);
    }
    removeItemAt(index) {
        const a = this.val;
        if (index >= 0 && index < a.length) {
            a.splice(index, 1);
            this.updateContexts();
        }
    }
    setLength(length) {
        const a = this.val;
        if (a.length != length) {
            a.length = length;
            this.updateContexts();
        }
    }
    sort(comparer) {
        this.val.sort(comparer);
        this.updateContexts();
    }
    modifyItems(action) {
        action(this.val);
        this.updateContexts();
    }
}
