export class tswRootEventHandler {
    htmlElement;
    eventHandlerDrivers = new Map();
    eventHandlers = new Map();
    constructor(htmlElement) {
        this.htmlElement = htmlElement;
    }
    attachElmEventHandler(el, elmEventMapItem) {
        let elmEventMapItems = this.eventHandlers.get(el);
        if (elmEventMapItems == null) {
            elmEventMapItems = [];
            this.eventHandlers.set(el, elmEventMapItems);
        }
        elmEventMapItems.push(elmEventMapItem);
        const ehd = this.getEventHandlerDriver(elmEventMapItem.eventType);
        ehd.attachEventListener(elmEventMapItem.eventName);
    }
    detachElmEventHandlers(el) {
        let elmHandlers = this.eventHandlers.get(el);
        if (elmHandlers) {
            elmHandlers.forEach(elmEventMapItem => {
                const ehd = this.getEventHandlerDriver(elmEventMapItem.eventType);
                ehd.detachEventListener(elmEventMapItem.eventName);
            });
        }
        this.eventHandlers.delete(el);
    }
    hasHandlers() {
        return this.eventHandlers.size > 0;
    }
    getEventHandlerDriver(eventType) {
        let ehd = this.eventHandlerDrivers.get(eventType);
        if (ehd == null) {
            const ehdType = tswEventHandlerDriver.DriverTypes.get(eventType);
            if (ehdType == null)
                throw new Error(`EventHandlerDriver is not found for eventType='${eventType}'`);
            ehd = new ehdType(this);
            this.eventHandlerDrivers.set(eventType, ehd);
        }
        return ehd;
    }
    findEventHandlers(htmlElement, eventName, eventType) {
        while (htmlElement && htmlElement != this.htmlElement) {
            const elmHandlersItems1 = this.eventHandlers.get(htmlElement);
            if (elmHandlersItems1) {
                const elmHandlersItems2 = elmHandlersItems1.filter(i => i.eventName == eventName);
                if (elmHandlersItems2.length > 0) {
                    const elmHandlersItems3 = elmHandlersItems2.filter(i => i.eventType == eventType);
                    return { htmlElement, elmHandlersItems: elmHandlersItems3 };
                }
            }
            htmlElement = htmlElement.parentNode;
        }
        return null;
    }
}
export class tswEventHandlerDriver {
    owner;
    static DriverTypes = new Map();
    attachedEventListeners = new Map();
    eventsListener = this.handleEvent.bind(this);
    constructor(owner) {
        this.owner = owner;
    }
    attachEventListener(eventName) {
        const count = this.attachedEventListeners.get(eventName) ?? 0;
        this.attachedEventListeners.set(eventName, count + 1);
        if (count == 0)
            this.addEventListener(eventName);
    }
    detachEventListener(eventName) {
        const count = this.attachedEventListeners.get(eventName) ?? 0;
        const countNew = count - 1;
        if (countNew > 0) {
            this.attachedEventListeners.set(eventName, countNew);
        }
        else {
            this.removeEventListener(eventName);
            this.attachedEventListeners.delete(eventName);
        }
    }
    getEventType() {
        throw new Error("Not implemented");
    }
    addEventListener(eventName) {
        throw new Error("Not implemented");
    }
    removeEventListener(eventName) {
        throw new Error("Not implemented");
    }
    handleEvent(e) {
        const target = e.target;
        if (!(target instanceof Element))
            return;
        const result = this.owner.findEventHandlers(target, e.type, this.getEventType());
        if (result == null)
            return;
        if (e.type == 'click' && result.htmlElement instanceof HTMLAnchorElement) {
            e.preventDefault();
        }
        for (const i of result.elmHandlersItems) {
            i.handleEvent.call(result.htmlElement, e, result.htmlElement);
        }
    }
}
export class tswEventHandlerDriverDom extends tswEventHandlerDriver {
    static EventType = 'dom';
    getEventType() {
        return tswEventHandlerDriverDom.EventType;
    }
    addEventListener(eventName) {
        this.owner.htmlElement.addEventListener(eventName, this.eventsListener);
    }
    removeEventListener(eventName) {
        this.owner.htmlElement.removeEventListener(eventName, this.eventsListener);
    }
}
tswEventHandlerDriver.DriverTypes.set(tswEventHandlerDriverDom.EventType, tswEventHandlerDriverDom);
